class App:
    """
    代码规范类
    """
    def __init__(self):
        self.window = None

    def bind_window(self):
        ...

    def bind_widgets(self):
        ...

    def register_for_event(self):
        ...

    def bind_poses(self):
        ...

    def run(self):
        self.window.run()

    def bind_layouts(self):
        ...

    def bind_pages(self):
        ...

    def bind_fonts(self):
        ...

    def process_widgets(self):
        ...

    def register_for_tasks(self):
        ...

    def bind_call_functions(self):
        ...

